-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- P6. Example tutorial. The lamp control of the flashing torche project
-- Plan: A synchronous FSM 
-- ref: http://digsys.upc.es/csd/P06/P6_T/Lamp/Lamp.html
--------------------------------------------------------------------------------

LIBRARY ieee;     
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;
			
ENTITY Lamp_Control IS
	Port (	
		CLK 	: IN	STD_LOGIC;
		ON_B	: IN	STD_LOGIC;
		CD   	: IN 	STD_LOGIC;
		States : OUT 	STD_LOGIC_VECTOR(2 DOWNTO 0);
		S 		: OUT	STD_LOGIC_VECTOR(1 DOWNTO 0)
	);
END Lamp_Control;

ARCHITECTURE FSM_like OF Lamp_Control IS

	TYPE State_type IS (Init_state, Button_pressed, Light_ON,
					Button_pressed_2, Light_flashing, Button_pressed_3);  

	SIGNAL current_state, next_state	: State_type ; 	
--------------------------------------------------------------------------------
-- Xilinx (clicking the ISE tap: "Design goals & strategies", file:default.xds)  
-- The legal values for this are "one_hot", "sequential", "johnson", 
-- "gray", and "auto".
--	ATTRIBUTE fsm_encoding : STRING;
--	ATTRIBUTE fsm_encoding of State_type : TYPE IS "johnson";
--
-- FSM encoding for Lattice 
-- ATTRIBUTE syn_encoding : STRING;
-- ATTRIBUTE syn_encoding OF State_type : TYPE IS "one-hot";


BEGIN

	-------- The state register that contains the memory cells
state_register: PROCESS (CD, CLK)
	BEGIN
						
	IF CD = '1' THEN 			-- reset counter
		current_state <= Init_state; 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous D-type flip-flops
		current_state <= next_state; 
	END IF;

END PROCESS state_register;

	-------- CC1: Combinational system for calculating next state
	-------- The truth table that contains all the state transitions (arrows)

CC_1: PROCESS (current_state, ON_B)
	BEGIN
	   
		CASE current_state IS
				
			WHEN Init_state =>
			   IF (ON_B = '1') THEN
			      	next_state <= Button_pressed; 
				ELSE    
					next_state <= Init_state;
				END IF;
				
			WHEN Button_pressed =>  
				IF (ON_B = '1') THEN
					next_state <= Button_pressed; 
				ELSE    
					next_state <= Light_ON ;
				END IF;
				
			WHEN Light_ON => 
			   IF (ON_B = '1') THEN
			      next_state <= Button_pressed_2; 
				ELSE    
					next_state <= Light_ON ;
				END IF;
				
			WHEN Button_pressed_2 => 
				IF (ON_B = '1') THEN
					next_state <= Button_pressed_2; 
				ELSE    
					next_state <= Light_flashing ;
				END IF;
				
			WHEN Light_flashing => 
			   IF (ON_B = '1') THEN
			      next_state <= Button_pressed_3; 
				ELSE    
					next_state <= Light_flashing ;
				END IF;

			WHEN Button_pressed_3 => 
				IF (ON_B = '1') THEN
					next_state <= Button_pressed_3; 
				ELSE    
					next_state <= Init_state ;
				END IF;
		
		END CASE ;		

END PROCESS CC_1;

	-------- CS_2: combinational system for calculating the outputs 
CC_2: PROCESS (current_state)
	BEGIN
		CASE current_state IS
			WHEN Init_state => 
				S <= "00";
-- To monitor the current state using the prototyping board LED's				
				States <= "000";
			WHEN Button_pressed => 
				S <= "01"; 	-- This Ch1 will set the output to '1' 
				States <= "001";
				
			WHEN Light_ON => 
				S <= "01";
				States <= "010";

			WHEN Button_pressed_2 => 
				S <= "10";	-- This Ch2 will set the intermittent output
				States <= "011";

			WHEN Light_flashing => 
				S <= "10";
				States <= "100";

			WHEN Button_pressed_3 => 
				S <= "00";
				States <= "101";

		END CASE ;		

END PROCESS CC_2;

-- Place here other logic if necessary



END FSM_like;